@@ -7,7 +7,7 @@ our @EXPORT_OK = qw(load_user validate_user);
 
 my $pkg = __PACKAGE__;
 my ($dbh, $sth, $app);
-has [qw(dbh sth app)];
+has [qw(dbh sth app plugin)];
 
 sub init {# from plugin! init Class vars
   my $self = shift;
@@ -20,23 +20,28 @@ sub init {# from plugin! init Class vars
   $sth = $self->sth
     or die "Нет STH";
   $self->app($self->{app} || $args{app});
+  $self->plugin($self->{plugin} || $args{plugin});
   $app = $self->app;
   return $self;
 }
 
 sub load_user {# import for Mojolicious::Plugin::Authentication
   my ($c, $uid) = @_;
-  my $u = $dbh->selectrow_hashref($sth->sth('user'), undef, ($uid, undef));
-  $c->app->log->debug("Loading user by id=$uid ". ($u ? 'success' : 'failed'));
-  $u->{pass} = '**********************' if $u;
-  return $u;
+  my $p = $dbh->selectrow_hashref($sth->sth('profile'), undef, ($uid, undef));
+  $c->app->log->debug("Loading profile by id=$uid ". ($p ? 'success' : 'failed'));
+  $p->{pass} = '**********************' if $p;
+  return $p;
 }
 
 sub validate_user {# import for Mojolicious::Plugin::Authentication
   my ($c, $login, $pass, $extradata) = @_;
-  if (my $u = $dbh->selectrow_hashref($sth->sth('user'), undef, (undef, $login))) {
-    return $u->{id}
-      if $u->{pass} eq $pass  && !$u->{disable};
+  
+  return $extradata->{id}
+    if $extradata && $extradata->{id};
+    
+  if (my $p = $dbh->selectrow_hashref($sth->sth('profile'), undef, (undef, $login))) {
+    return $p->{id}
+      if $p->{pass} eq $pass  && !$p->{disable};
   }
   return undef;
 }
@@ -57,7 +62,9 @@ sub apply_route {# meth in Plugin
     and return undef
     if $r_hash->{disable};
   
-  $app->log->debug("Skip route id=[$r_hash->{id}] empty request")
+  $r_hash->{request} //= $r_hash->{route};
+  
+  $app->log->debug("Skip route @{[$app->dumper($r_hash) =~ s/\s+//gr]}: empty request")
     and return undef
     unless $r_hash->{request};
   
@@ -66,7 +73,8 @@ sub apply_route {# meth in Plugin
     if $r_hash->{request} =~ /^#/;
   
   my @request = grep /\S/, split /\s+/, $r_hash->{request}
-    or return;
+    or $app->log->debug("Skip route @{[$app->dumper($r_hash) =~ s/\s+//gr]}: bad request")
+    and return;
   my $nr = $r->route(pop @request);
   $nr->via(@request) if @request;
   
@@ -75,13 +83,21 @@ sub apply_route {# meth in Plugin
   # STEP ACCESS
   $nr->over(access => $r_hash);
   
-  if ($r_hash->{controller}) {
-    $nr->to(controller=>$r_hash->{controller}, action => $r_hash->{action},  $r_hash->{namespace} ? (namespace => $r_hash->{namespace}) : (),);
-  } elsif ($r_hash->{callback}) {
+  if ( $r_hash->{action} ) {
+
+    my %ns = (namespace => $r_hash->{namespace})
+      if $r_hash->{namespace};
+      
+    if ( $r_hash->{action} =~ /#/ ) { $nr->to($r_hash->{action}, %ns); }
+    else { $nr->to(controller=>$r_hash->{controller}, action => $r_hash->{action}, %ns,); }
+    
+  } elsif ( $r_hash->{callback} ) {
+    
     my $cb = eval $r_hash->{callback};
     die "Compile error on callback: [$@]", $app->dumper($r_hash)
       if $@;
     $nr->to(cb => $cb);
+    
   } else {
     die "No defaults for route: ", $app->dumper($r_hash);
   }
@@ -97,8 +113,8 @@ sub db_routes {
 
 sub load_user_roles {
   my ($self, $user) = @_;
-  $user->{roles} ||= $dbh->selectall_arrayref($sth->sth('user roles'), { Slice => {} }, ($user->{id}))
-    and $app->log->debug("Loading user roles:", $app->dumper($user->{roles}) =~ s/\s+//gr,);
+  $user->{roles} ||= $dbh->selectall_arrayref($sth->sth('profile roles'), { Slice => {} }, ($user->{id}))
+    and $app->log->debug("Loading profile roles:", $app->dumper($user->{roles}) =~ s/\s+//gr,);
   
 }
 
@@ -172,7 +188,7 @@ There are two ways of flow: explicit and implicit.
 
 =item * Explicit access
 
-Check by tables ids: routes, actions, controllers, namespaces. Check refs to user roles ids.
+Check by tables ids: routes, actions, controllers, namespaces. Check refs to profile roles ids.
 
 =item * Implicit access
 
@@ -206,6 +222,15 @@ See detail L<Mojolicious::Plugin::RoutesAuthDBI#access>
 
 Both above options determining the module which will play as manager of authentication, accessing and generate routing from DBI source.
 
+=item * B<pos> - hashref, default:
+
+  pos => {
+    namespace => 'Mojolicious::Plugin::RoutesAuthDBI',
+    module => 'POS::Access',
+  },
+
+SQL-dictionary for DBI statements. See L<Mojolicious::Plugin::RoutesAuthDBI::POS::Access>.
+
 =item * B<fail_auth_cb> = sub {my $c = shift;...}
 
 This callback invoke when request need auth route but authentication was failure.
@@ -220,9 +245,9 @@ This callback invoke when request need auth route but access was failure. $route
 
 =over 4
 
-=item * B<load_user($c, $uid)> - fetch user record from table users by COOKIES. Import for Mojolicious::Plugin::Authentication. Required.
+=item * B<load_user($c, $uid)> - fetch user record from table profiles by COOKIES. Import for Mojolicious::Plugin::Authentication. Required.
 
-=item * B<validate_user($c, $login, $pass, $extradata)> - fetch user record from table users by Mojolicious::Plugin::Authentication. Required.
+=item * B<validate_user($c, $login, $pass, $extradata)> - fetch login record from table logins by Mojolicious::Plugin::Authentication. Required. If hashref $extradata->{id} then no fetch and $extradata->{id} will return.
 
 =back
 
@@ -256,19 +281,19 @@ Check access to route ($id1 arrayref - either route id or action id or controlle
 
 =item * B<access_namespace($namespace, $id2)>
 
-Check implicit access to route by $namespace for user roles ids ($id2 arrayref). Must return false for deny access or true - allow access to all actions of this namespace.
+Check implicit access to route by $namespace for profile roles ids ($id2 arrayref). Must return false for deny access or true - allow access to all actions of this namespace.
 
 =item * B<access_controller($namespace, $controller, $id2)>
 
-Check implicit access to route by $namespace and $controller for user roles ids ($id2 arrayref). Must return false for deny access or true - allow access to all actions of this controller.
+Check implicit access to route by $namespace and $controller for profile roles ids ($id2 arrayref). Must return false for deny access or true - allow access to all actions of this controller.
 
 =item * B<access_action($namespace, $controller, $action, $id2)>
 
-Check implicit access to route by $namespace and $controller and $action for user roles ids ($id2 arrayref). Must return false for deny access or true - allow access to this action.
+Check implicit access to route by $namespace and $controller and $action for profile roles ids ($id2 arrayref). Must return false for deny access or true - allow access to this action.
 
 =item * B<access_role($role, $id2)>
 
-Check implicit access to route by $role (id|name) and user roles ids ($id2 arrayref). Must return false for deny access or true - allow access.
+Check implicit access to route by $role (id|name) and profile roles ids ($id2 arrayref). Must return false for deny access or true - allow access.
 
 =back
 
