@@ -80,6 +80,8 @@ has config => sub {# только $Init !
   merge $self->{providers}, $self->_providers;
 };
 
+has auth_profile => sub { shift->${ \$Init->plugin->merge_conf->{auth}{current_user_fn} };};
+
 has ua => sub {shift->app->ua->connect_timeout(30);};
 
 sub init {# from plugin
@@ -124,9 +126,7 @@ sub login {
     die "OAuth provider [$site_name] does not configured: [@fatal] is not defined";
   }
   
-  #~ my $fail_auth_cb = $Init->{fail_auth_cb};
-  
-  my $auth_profile = $c->${ \$Init->plugin->merge_conf->{auth}{current_user_fn} };
+  my $auth_profile = $c->auth_profile;
   
   my $r; $r = $dbh->selectrow_hashref($sth->sth('check profile oauth'), undef, ($auth_profile->{id}, $site->{id}))
     and $c->app->log->warn("Попытка двойной авторизации сайта $site_name", $c->dumper($r), "профиль: ", $c->dumper($auth_profile),)
@@ -202,6 +202,23 @@ sub login {
   
 }
 
+sub отсоединить {
+  my $c = shift;
+  my $site_name = $c->stash('site');
+
+  my $site = $c->oauth2->providers->{$site_name}
+    or die "No such oauth provider [$site_name]" ;
+  
+  my $auth_profile = $c->auth_profile;
+  
+  my $r = $dbh->selectrow_hashref($sth->sth('отсоединить oauth'), undef, ($site->{id}, $auth_profile->{id},));
+  #~ $c->app->log->debug("Убрал авторизацию сайта [$site_name] профиля [$auth_profile->{id}]", $c->dumper($r));
+  
+  $dbh->selectrow_hashref($Init->plugin->admin->sth->sth('del ref'), undef, ($r->{ref_id}, undef, undef));
+  
+  $c->redirect_to($c->param('redirect') || 'profile');
+}
+
 
 sub out {# выход
   my $c = shift;
@@ -220,6 +237,13 @@ sub _routes {# from plugin!
     action => 'login',
     name => 'oauth-login',
   },
+  {request=>'/detach/:site',
+    namespace=>$Init->{namespace},
+    controller=>$Init->{controller} || $Init->{module},
+    action => 'отсоединить',
+    name => 'oauth-detach',
+    auth=>'only',
+  },
   {request =>'/logout',
     namespace=>$Init->{namespace},
     controller=>$Init->{controller} || $Init->{module},
@@ -279,7 +303,7 @@ See L<https://github.com/mche/Mojolicious-Plugin-RoutesAuthDBI/blob/master/Diagr
 =item * B<controller> - module controller name, default 'OAuth2',
 
 
-=item * B<providers> - hashref. required.
+=item * B<providers> - hashref. Required.
 
   providers => {google=>{key=> ..., secret=>..., }, ...},
 
@@ -336,7 +360,7 @@ disable oauth module
 
 This oauth controller routes. Return array of hashrefs routes records for apply route on app. Plugin internal use.
 
-=head1 Routes
+=head1 Controller routes
 
 =head2 /login/:site
 
@@ -344,11 +368,15 @@ Main route of this controller. Stash B<site> is the name of the hash key of the
 
   <a href="<%= $c->url_for('oauth-login', site=> 'google')->query(redirect=>'profile') %>">Login by google</a>
 
-This route has name 'oauth-login'. This route accept param 'redirect' and will use for $c->redirect_to after success oauith and also failed oauth with out param 'err'.
+This route has builtin name 'oauth-login'. This route accept param 'redirect' and will use for $c->redirect_to after success oauith and also failed oauth clauses with param 'err'.
+
+=head2 /detach/:site
+
+Remove attached oauth user to profile. Stash B<site> and param 'redirect' as above. Route has builtin name 'oauth-detach'.
 
 =head2 /logout
 
-Clear session and redirect to param 'redirect' || '/'. This route has name 'logout'.
+Clear session and redirect to param 'redirect' || '/'. This route has builtin name 'logout'. 
 
 =head1 SEE ALSO
 
