@@ -138,6 +138,23 @@ L<DBIx::POS::Template>
   
   where p.id=? and o.site_id=?
 
+=head2 отсоединить oauth
+
+=name отсоединить oauth
+
+=desc
+
+=sql
+
+  delete from "{% $schema %}"."{% $tables{oauth_users} %}" d
+  using "{% $schema %}"."{% $tables{refs} %}" r
+  where d.site_id = ?
+    and r.id1=? -- ид профиля
+    and d.id=r.id2
+  
+  RETURNING d.*, r.id as ref_id;
+
+
 =cut
 
 1;
\ No newline at end of file
